﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using SmoLiteApi.Sql;
using System.Data.SqlClient;
using SmoLiteApi.Helper;

namespace SmoLiteApi
{
    /// <summary/>
    public class SchemaCollection : IEnumerable<Schema>
    {
        private Database _database;

        /// <summary/>
        internal SchemaCollection(Database database)
        {
            this._database = database;
        }

        /// <summary/>
        public Schema this[string name]
        {
            get
            {
                // get the list
                List<Schema> schemas = this.FetchObjects(name);

                // return the data
                if (schemas.Count == 0)
                {
                    return null;
                }

                // return the one and only
                return schemas[0];
            }
        }

        /// <summary/>
        public IEnumerator<Schema> GetEnumerator()
        {
            return this.FetchObjects(null).GetEnumerator();
        }

        /// <summary/>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.FetchObjects(null).GetEnumerator();
        }

        /// <summary/>
        private List<Schema> FetchObjects(string name)
        {
            List<Schema> schemas = new List<Schema>();

            SchemaSelectSql schemaSelectSql = new SchemaSelectSql();

            // set attributes
            schemaSelectSql.AddObject(new { Name = name });

            using (SqlCommandHelper sqlCommandHelper = SqlCommandHelper.FetchSqlCommand(this._database.SqlConnection, schemaSelectSql.Render()))
            {
                using (SqlDataReader sqlDataReader = sqlCommandHelper.ExecuteReader())
                {
                    while (sqlDataReader.Read())
                    {
                        Schema schema = new Schema(
                            this._database,
                            sqlDataReader.GetString(sqlDataReader.GetOrdinal(Constant.PARAMETER_NAME)));

                        // add to the list
                        schemas.Add(schema);
                    }
                }
            }

            // retun the list
            return schemas;
        }
    }
}
